/*  muzikQ - Curses based audio player
 *
 *  Copyright (C) 2008 Tim A. Brown
 * 
 *  This file is part of muzikQ.

    muzikQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    muzikQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with muzikQ.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "muzikQ.h"
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>

int row, col, rows;

/* int strpos( char *ptr, char chr )
{
  int pos = 0;
  char *p=ptr;
  while( p[pos] )
  {
    if( p[pos] != chr )
    {
      pos++;
    }
    else
    {
      break;
    }
  }
  return( pos );
} */


void init_screen()
  {
        initscr();
        nocbreak();
        keypad(stdscr, TRUE);
        noecho();
        halfdelay (1);
	curs_set (0);
	if (has_colors ())
	  {
	    start_color ();
            init_pair (1, COLOR_WHITE, COLOR_GREEN);
	    init_pair (2, COLOR_BLACK, COLOR_BLACK);
	    init_pair (3, COLOR_WHITE, COLOR_BLACK);
	    init_pair (4, COLOR_GREEN, COLOR_BLACK);
            init_pair (5, COLOR_RED, COLOR_BLACK);
            init_pair (6, COLOR_WHITE, COLOR_WHITE);
	  }
	getmaxyx(stdscr,row,col);
	main_win = newwin(5, col, 0, 0);
        wbkgd (main_win, ' ' | COLOR_PAIR (2) | A_BOLD);
        getmaxyx(stdscr,row,col);
        left_win = newwin(row - 5, col / 3, 5, 0);
        wbkgd (left_win, ' ' | COLOR_PAIR (2) | A_BOLD);
        getmaxyx(stdscr,row,col);
        right_win = newwin(row - 5, col - (col / 3), 5, col / 3);
        wbkgd (right_win, ' ' | COLOR_PAIR (2) | A_BOLD);
        leaveok (right_win, FALSE);
        leaveok (left_win, FALSE);
        leaveok (main_win, FALSE);
	typeahead (-1);
        refresh();
        doupdate();
  }

void show_main_win(float current_time)
  {
	char time_string[14];
        
        werase(main_win);
        getmaxyx(stdscr,row,col);
        box(main_win, 0, 0);
	wattr_on (main_win, COLOR_PAIR (3), NULL);
	mvwprintw (main_win, 0, col - 14, "muzikQ %s", VERSION);
	wattr_on (main_win, COLOR_PAIR (2) | A_BOLD, NULL);
	mvwprintw (main_win, 4, 2, "[      ]");
        if (random_play)
            wattr_on (main_win, COLOR_PAIR(3) | A_BOLD, NULL);
        mvwprintw (main_win, 4, 3, "Random");
        wattr_on (main_win, COLOR_PAIR (2) | A_BOLD, NULL);
	mvwprintw (main_win, 4, 11, "[    ]");
        if (repeat_play)
            wattr_on (main_win, COLOR_PAIR(3) | A_BOLD, NULL);
	mvwprintw (main_win, 4, 12, "Loop");
        wattr_on (main_win, COLOR_PAIR (2) | A_BOLD, NULL);
	mvwprintw (main_win, 1, col - 16, "Vol:");
	wattr_on (main_win, COLOR_PAIR (2) | A_BOLD, NULL);
	wattr_off(main_win, A_BOLD, NULL);
	wattr_on (main_win, COLOR_PAIR (3), NULL);
	if (strlen (play[current_song].artist) == 0 && strlen (play[current_song].title) == 0)
	  mvwprintw (main_win, 1, 2, "%s", play[current_song].mfile);
        else
	  mvwprintw (main_win, 1, 2, "%s - %s", play[current_song].artist, play[current_song].title);
	wattr_on (main_win, COLOR_PAIR (2) | A_BOLD, NULL);
	mvwprintw (main_win, 2, col - 17, " Rating: ( )");
	wattr_on (main_win, COLOR_PAIR (4) | A_BOLD, NULL);
	mvwprintw (main_win, 2, col - 7, "%i", play[current_song].rate);
	wattr_on (main_win, COLOR_PAIR(2), NULL);
	if (play[current_song].channels > 1)
	  {
	      mvwprintw (main_win, 2, 2, "(%s %ikbit/s %i stereo)", play[current_song].type, play[current_song].bitrate, play[current_song].sample);
	  } else mvwprintw (main_win, 2, 2, "(%s %ikbit/s %i mono)", play[current_song].type, 			play[current_song].bitrate, play[current_song].sample);

	wattr_on (main_win, COLOR_PAIR (3), NULL);
	sprintf (time_string,
	   "%d:%.2d / %d:%.2d",
	   ((int) current_time / 60),
	   ((int) current_time -
	    (int) ((int) current_time /
		   60) * 60),
            ((int) play[current_song].seconds / 60),
            ((int) play[current_song].seconds - (int) ((int) play[current_song].seconds / 60) *60));
	mvwprintw (main_win, 3, (col - 2) - 8 - strlen (time_string), "     %s", time_string);
	wattr_on (main_win, COLOR_PAIR (1), NULL);
	mvwhline (main_win, 1, col - 11, 32, vol / 10);
	mvwhline (main_win, 3, 2, 32, current_time / play[current_song].seconds * (col - 10) -14);
        wattr_on (main_win, COLOR_PAIR (2), NULL);
	wrefresh(main_win);
        doupdate();
  }

void show_left_win(int song_num)
  {  
        int total_h, total_m, total_s;
	float size;
        char unit[3], *tmpstr;
        int lrow, lcol, pos, x, z, cline;
        
        getmaxyx(left_win, lrow, lcol);
        char tfn[lcol];
        if (total_size > 1048576)
	   {
	      size = (float) total_size / 1048576;
	      strcpy ((char *) &unit, "Gb");
	   }
	else
	   {
	      size = (float) total_size / 1024;
	      strcpy ((char *) &unit, "Mb");
	   }
	total_h = (int) total_time / 3600;
	total_m = (int) (total_time - (total_h * 3600)) / 60;
	total_s = total_time - (total_h * 3600) - (total_m * 60);

        werase(left_win);
        getmaxyx(stdscr, row, col);
        box(left_win, 0, 0);
	wattr_on (left_win, COLOR_PAIR (3), NULL);
	mvwprintw (left_win, 0, 2, "Tag Info");
	mvwprintw (left_win, 1, 2, "Format  : ");
	mvwprintw (left_win, 1, 12, "%s", play[song_num].type);
        mvwprintw (left_win, 2, 2, "Artist  : ");
	mvwprintw (left_win, 2, 12, "%s", play[song_num].artist);
	mvwprintw (left_win, 3, 2, "Title   : ");
	mvwprintw (left_win, 3, 12, "%s", play[song_num].title);
	mvwprintw (left_win, 4, 2, "Album   : ");
	mvwprintw (left_win, 4, 12, "%s", play[song_num].album);
	mvwprintw (left_win, 5, 2, "Genre   : ");
	mvwprintw (left_win, 5, 12, "%s", play[song_num].genre);
	mvwprintw (left_win, 6, 2, "Year    : ");
	mvwprintw (left_win, 6, 12, "%s", play[song_num].year);
	mvwprintw (left_win, 7, 2, "Track   : ");
	mvwprintw (left_win, 7, 12, "%i", play[song_num].track);
	mvwprintw (left_win, 8, 2, "Channels: ");
	mvwprintw (left_win, 8, 12, "%i", play[song_num].channels);
	mvwprintw (left_win, 9, 2, "Length  : ");
	mvwprintw (left_win, 9, 12,  "%.2d:%.2d",
			 ((int) play[song_num].seconds / 60),
			 ((int) play[song_num].seconds -
			 (int) ((int) play[song_num].seconds / 60) * 60));
	mvwprintw (left_win, 10, 2, "Rating  : ");
	mvwprintw (left_win, 10, 12, "%i", play[song_num].rate);
	mvwprintw (left_win, 11, 2, "Sample  : ");
	mvwprintw (left_win, 11, 12, "%i", play[song_num].sample);
	mvwprintw (left_win, 12, 2, "Bitrate : ");
	mvwprintw (left_win, 12, 12, "%i", play[song_num].bitrate);
        mvwprintw (left_win, 13, 2, "Gain    : ");
        if (play[song_num].vol > 0)
            mvwprintw (left_win, 13, 12, "%i", play[song_num].vol);
        else mvwprintw (left_win, 13, 12, "%i", vol);
	mvwprintw (left_win, 14, 2, "Filesize: ");
	mvwprintw (left_win, 14, 12, "%i", play[song_num].size);
	mvwprintw (left_win, 15, 2, "Filename: ");
        if (strlen(play[song_num].mfile) >= lcol -13)
            {
                strncpy(tfn, play[song_num].mfile, lcol - 15);
                tfn[lcol - 16] = '\0';
                mvwprintw (left_win, 15, 12, "%s..", tfn);
            }
        else 
            {
                mvwprintw (left_win, 15, 12, "%s", play[song_num].mfile);
            }
	mvwprintw (left_win, 16, 2, "Comment : ");
        cline = 16;
        if (strlen(play[song_num].comment) >= lcol -13)
            {
                strncpy(tfn, play[song_num].comment, lcol - 15);
                tfn[lcol - 16] = '\0';
                mvwprintw (left_win, 16, 12, "%s..", tfn);
            }
        else 
            {
                mvwprintw (left_win, 16, 12, "%s", play[song_num].comment);
            }
        wattr_on (left_win, COLOR_PAIR (3), NULL);
        if (use_cdrom)
            mvwprintw (left_win, row - 7, 2, "Total: %d files, %.1f %s, %dh %.2dm %.2ds", 
                    num_songs - cdrom->numtracks, size, unit, total_h, total_m, total_s);
        else mvwprintw (left_win, row - 7, 2, "Total: %d files, %.1f %s, %dh %.2dm %.2ds", 
                    num_songs, size, unit, total_h, total_m, total_s);
        wattr_on (left_win, COLOR_PAIR (2), NULL);
	wrefresh(left_win);
	doupdate();
  }

void show_right_win()
  {
	int x;
        
        werase(right_win);
        getmaxyx(stdscr, row, col);
        rows = (row - 7);
	
	if (selected >= pagetop + rows)
	  pagetop += rows - 2;
	else if (selected < pagetop)
	  pagetop -= rows - 2;
	if (pagetop < 0)
	  pagetop = 0;
	else if (pagetop >= num_songs)
	  pagetop = num_songs - 1;      
        wattr_on (right_win, COLOR_PAIR (3), NULL);
	mvwprintw (right_win, 0, 2, "Playlist");
	for (x=0; x < rows; x++)
         {
        if (x + pagetop < num_songs)
            {
                if (x + pagetop == selected) 
                   {
                        wattr_on (right_win, COLOR_PAIR (4), NULL); 
                   }
                else {
                        wattr_on (right_win, COLOR_PAIR (3), NULL); 
                     }
            }
          mvwhline (right_win,x + 1, 2, 32, col - 2);
	  if (!strlen(play[x+pagetop].mfile) == 0)  
          {  
            if (strlen (play[x+pagetop].artist) == 0 && strlen (play[x+pagetop].title) == 0)
                mvwprintw (right_win, x + 1, 2, "%s", play[x+pagetop].mfile);
            else
                mvwprintw (right_win, x + 1, 2, "%s - %s", play[x+pagetop].artist, play[x+pagetop].title);
	    if (play[x].seconds)
	      {
		  mvwprintw (right_win, x + 1, col - (col / 3) - 8,
			  "%.2d:%.2d",
			  ((int) play[x+pagetop].seconds / 60),
			  ((int) play[x+pagetop].seconds -
			  (int) ((int) play[x+pagetop].seconds / 60) * 60));
	      }
	    mvwprintw (right_win, x + 1, col - (col / 3) - 12, "(%d)", play[x+pagetop].rate);
          } 
        }
	for (x=1; x < row -1; x++)
	  {
	    wattr_on (right_win, COLOR_PAIR (1), NULL);
	    mvwprintw (right_win, x, (col - (col / 3)) - 2, " ");
	  }
        wattr_on (right_win, COLOR_PAIR (6), NULL);
        mvwprintw (right_win, (((int) ((float) selected / (num_songs -1) * rows) + 1)), (col -(col / 3)) - 2, " ");
        show_left_win(selected);
        wattr_on (right_win, COLOR_PAIR (2), NULL);
        box(right_win, 0, 0);
	wrefresh(right_win);
        doupdate();
  }

void update_scrollbar ()
{
    rows = (row - 8);
    wattr_on (right_win, COLOR_PAIR (6), NULL); 
    mvwvline (right_win, 
        (int) ((float) selected / num_songs * rows) + 1, (col -(col / 3) - 2), 32, 1);
    doupdate();
    wattr_on (right_win, COLOR_PAIR (2), NULL); 
}

void show_tag_win ()
{
  WINDOW *tag_win;
  char artist[31], title[31];
  char comment[31], album[31];
  char genre[31], year[5];
  int answer, track, new_track;
  char *commentp, *artistp, *titlep, *yearp, *genrep, *albump, *trackp;
  
  strcpy (artist, play[selected].artist);
  strcpy (title, play[selected].title);
  strcpy (comment, play[selected].comment);
  strcpy (album, play[selected].album);
  strcpy (year, play[selected].year);
  strcpy (genre, play[selected].genre);
  track = play[selected].track;
  artistp = artist;
  titlep = title;
  commentp = comment;
  yearp = year;
  albump = album;
  genrep = genre;
  trackp = itoa(track);
  getmaxyx(stdscr, row, col);
  tag_win = newwin (row - 2, col - 10, 2, 5);
  curs_set (1);
  echo ();
  werase (tag_win);
  wbkgd (tag_win, COLOR_PAIR (2) | A_BOLD);
  box (tag_win, 0, 0);
  mvwprintw (tag_win, 2, 2, "Artist:");
  mvwprintw (tag_win, 4, 2, "Title:");
  mvwprintw (tag_win, 6, 2, "Album:");
  mvwprintw (tag_win, 8, 2, "Genre:");
  mvwprintw (tag_win, 10, 2, "Year:");
  mvwprintw (tag_win, 12, 2, "Track:");
  mvwprintw (tag_win, 14, 2, "Comment:");
  
  wattr_on (tag_win, COLOR_PAIR (3), NULL);
  mvwhline (tag_win, 2, 10, 32, 31);
  mvwhline (tag_win, 4, 10, 32, 31);
  mvwhline (tag_win, 6, 10, 32, 31);
  mvwhline (tag_win, 8, 10, 32, 31);
  mvwhline (tag_win, 10, 10, 32, 31);
  mvwhline (tag_win, 12, 10, 32, 31);
  mvwhline (tag_win, 14, 10, 32, 31);
  
  wrefresh (tag_win);
  
  mvwgetnstr (tag_win, 2, 10, artistp, 30);
  if (strlen(artistp) == 0)
      mvwprintw (tag_win, 2, 10, "%s", play[selected].artist);
  mvwgetnstr (tag_win, 4, 10, titlep, 30);
  if (strlen(titlep) == 0)
      mvwprintw (tag_win, 4, 10, "%s", play[selected].title);
  mvwgetnstr (tag_win, 6, 10, albump, 30);
  if (strlen(albump) == 0)
      mvwprintw (tag_win, 6, 10, "%s", play[selected].album);
  mvwgetnstr (tag_win, 8, 10, genrep, 30);
  if (strlen(genrep) == 0)
      mvwprintw (tag_win, 8, 10, "%s", play[selected].genre);
  mvwgetnstr (tag_win, 10, 10, yearp, 5);
  if (strlen(yearp) == 0)
      mvwprintw (tag_win, 10, 10, "%s", play[selected].year);
  mvwgetnstr (tag_win, 12, 10, trackp, 2);
  if (trackp <= 0)
      mvwprintw (tag_win, 12, 10, "%i", play[selected].track);
  mvwgetnstr (tag_win, 14, 10, commentp, 30);
  if (strlen(commentp) == 0)
      mvwprintw (tag_win, 14, 10, "%s", play[selected].comment);
  
  new_track = atoi(trackp);
  
  wattr_on (tag_win, COLOR_PAIR (2), NULL);
  mvwprintw (tag_win, 16, 2, "Are you sure? [Y/n]");
  curs_set (0);
  noecho ();
  nodelay (tag_win, FALSE);
  notimeout (tag_win, TRUE);
  answer = wgetch (tag_win);
      
  if (toupper(answer) != 'N' && answer != 10)
      {
      if (strlen(artistp) != 0)
        strcpy (play[selected].artist, artistp);
      if (strlen(titlep) != 0)     
        strcpy (play[selected].title, titlep);
      if (strlen(commentp) != 0)
        strcpy (play[selected].comment, commentp);
      if (strlen(albump) != 0)     
        strcpy (play[selected].album, albump);
      if (strlen(yearp) != 0)
        strcpy (play[selected].year, yearp);
      if (strlen(genrep) != 0)
        strcpy (play[selected].genre, genrep);
      if (new_track > 0)
        play[selected].track = new_track;
      saveTags(selected);
      }
 
  halfdelay (1);
  delwin (tag_win);
  doupdate();
}

int show_playlist_win ()
{
  WINDOW *mkQ_win;
  char filename[100];
  int answer;
  getmaxyx(stdscr, row, col);
  mkQ_win = newwin (6, col - 10, 6 + ((row - 17) / 2),
          (col - (col - 10)) / 2);
  curs_set (1);
  echo ();
  werase (mkQ_win);
  wbkgd (mkQ_win, COLOR_PAIR (2) | A_BOLD);
  box (mkQ_win, 0, 0);
  mvwprintw (mkQ_win, 2, 2, "Filename:");
  mvwprintw (mkQ_win, 2, col - 16, ".mkq");
  wattr_on (mkQ_win, COLOR_PAIR (3), NULL);
  mvwhline (mkQ_win, 2, 12, 32, col - 28);
  wrefresh (mkQ_win);
  mvwgetnstr (mkQ_win, 2, 12, (char *) &filename, col - 28);
  if (strlen (filename) != 0)
    {
        wattr_on (mkQ_win, COLOR_PAIR (2), NULL);
        mvwprintw (mkQ_win, 4, 2, "Are you sure? [Y/n]");
        curs_set (0);
        noecho ();
        nodelay (mkQ_win, FALSE);
        notimeout (mkQ_win, TRUE);
        answer = wgetch (mkQ_win);
    }
  halfdelay (1);
  delwin (mkQ_win);
  if (strlen(filename) != 0 && answer != 'n' && answer != 'N')
    {
      sprintf ((char *) &mkQplaylist, "%s.mkq", filename);
      return (1);
    }
  return 0;
  doupdate();
}

int confirm (char *statement, char *question)
{
  WINDOW *confirm_win;
  int answer, rrow, rcol;
  int width, height;
  if (strlen (statement))
    {
      if (strlen (statement) > strlen (question) + 6)
	width = strlen (statement) + 4;
      else
	width = strlen (question) + 10;
      height = 6;
    }
  else
    {
      width = strlen (question) + 10;
      height = 5;
    }
  getmaxyx(stdscr, row, col);
  confirm_win = newwin (height, width, 7 + ((row - 7 - height) / 2),
          (col - width) / 2);
  werase (confirm_win);
  wbkgd (confirm_win, COLOR_PAIR (2) | A_BOLD);
  box (confirm_win, 0, 0);
  wattr_on (confirm_win, COLOR_PAIR (2), NULL);
  if (strlen (statement))
    mvwprintw (confirm_win, 2, 2, "%s", statement);
  mvwprintw (confirm_win, height - 3, 2, "%s [Y/n]", question);
  nodelay (confirm_win, FALSE);
  notimeout (confirm_win, TRUE);
  answer = wgetch (confirm_win);
  halfdelay (1);
  delwin (confirm_win);
  if (answer != 'n' && answer != 'N')
    {
      return (1);
    }
  return (0);
  doupdate();
}

char *stristr (const char *String, const char *Pattern)
{
  char *pptr, *sptr, *start;
  unsigned int slen, plen;

  for (start = (char *) String,
       pptr = (char *) Pattern,
       slen = strlen (String), plen = strlen (Pattern);
       /* while string length not shorter than pattern length */
       slen >= plen; start++, slen--)
    {
      /* find start of pattern in string */
      while (toupper (*start) != toupper (*Pattern))
	{
	  start++;
	  slen--;

	  /* if pattern longer than string */

	  if (slen < plen)
	    return (NULL);
	}

      sptr = start;
      pptr = (char *) Pattern;

      while (toupper (*sptr) == toupper (*pptr))
	{
	  sptr++;
	  pptr++;

	  /* if end of pattern then pattern was found */

	  if ('\0' == *pptr)
	    return (start);
	}
    }
  return (NULL);
}

void show_search_win ()
{
  WINDOW *search_win;
  char *found1;
  char *found2;
  char *found3;
  int cursong;
  char newsearchfor[27];
  getmaxyx(stdscr, row, col);
  search_win =
          newwin (6, 43, 8 + ((row - 17) / 2), (col - 43) / 2);
  curs_set (1);
  echo ();
  werase (search_win);
  wbkgd (search_win, COLOR_PAIR (2) | A_BOLD);
  box (search_win, 0, 0);
  mvwprintw (search_win, 2, 2, "Search for:");
  if (strlen (searchfor))
    mvwprintw (search_win, 3, 2, "[ %s ]", searchfor);
  wattr_on (search_win, COLOR_PAIR (3), NULL);
  mvwhline (search_win, 2, 14, 32, 27);
  wrefresh (search_win);
  mvwgetnstr (search_win, 2, 14, newsearchfor, 26);
  if (strlen (newsearchfor))
    strcpy (searchfor, newsearchfor);
  if (strlen (searchfor))
    {
      for (cursong = selected + 1; cursong < num_songs; cursong++)

	{
	  found1 = stristr (play[cursong].mfile, searchfor);
	  found2 = stristr (play[cursong].artist, searchfor);
	  found3 = stristr (play[cursong].title, searchfor);
	  if (found1 != NULL || found2 != NULL || found3 != NULL)

	    {
	      selected = cursong;
	      break;
	    }
	}
      if (selected != cursong)
	for (cursong = 0; cursong < selected; cursong++)

	  {
	    found1 = stristr (play[cursong].mfile, searchfor);
	    found2 = stristr (play[cursong].artist, searchfor);
	    found3 = stristr (play[cursong].title, searchfor);
	    if (found1 != NULL || found2 != NULL || found3 != NULL)

	      {
		selected = cursong;
		break;
	      }
	  }
      pagetop = selected - (row - 9) / 2;
    }
  curs_set (0);
  noecho ();
  wclear(stdscr);
  delwin (search_win);
  repaint();
  doupdate();
}

void show_help ()
{
  int help_rows = 30;
  struct help_struct
  {
    char key[12];
    char desc[40];
  }
  help[help_rows];
  WINDOW *help_win;
  int rows;
  int c, i, current = 0;

  strcpy (help[0].key, "LEFT");
  strcpy (help[0].desc, "Seek left within song");
  strcpy (help[1].key, "RIGHT");
  strcpy (help[1].desc, "Seek right within song");
  strcpy (help[2].key, "UP");
  strcpy (help[2].desc, "Move up");
  strcpy (help[3].key, "DOWN");
  strcpy (help[3].desc, "Move down");
  strcpy (help[4].key, "PGUP");
  strcpy (help[4].desc, "Jump up");
  strcpy (help[5].key, "PGDN");
  strcpy (help[5].desc, "Jump down");
  strcpy (help[6].key, "ENTER");
  strcpy (help[6].desc, "Select song");
  strcpy (help[7].desc, "Pause/unpause");
  strcpy(help[7].key,"SPACE");
  strcpy(help[8].key,"+");
  strcpy (help[8].desc, "Increase volume for current song");
  strcpy(help[9].key,"-");
  strcpy (help[9].desc, "Decrease volume for current song");
  strcpy(help[10].key,"N");
  strcpy (help[10].desc, "Next song");
  strcpy(help[11].key,"/");
  strcpy (help[11].desc, "Search within the playlist");
  strcpy(help[12].key,"F1");
  strcpy (help[12].desc, "Sort playlist according to Artist");
  strcpy(help[13].key,"F2");
  strcpy (help[13].desc, "Sort playlist according to Song title");
  strcpy(help[14].key,"F3");
  strcpy (help[14].desc, "Sort playlist according to Time");
  strcpy(help[15].key,"F4");
  strcpy (help[15].desc, "Sort playlist according to Rating");
  strcpy (help[16].key, "1-9");
  strcpy (help[16].desc, "Set rating of selected song");
  strcpy(help[17].key,"A");
  strcpy (help[17].desc, "Add files to playlist");
  strcpy(help[18].key,"R");
  strcpy (help[18].desc, "Enable/disable random play mode");
  strcpy(help[19].key,"L");
  strcpy (help[19].desc, "Enable/disable loop play mode");
  strcpy(help[20].key,"E");
  strcpy (help[20].desc, "Edit tags for selected song");
  strcpy(help[21].key,"S");
  strcpy (help[21].desc, "Save playlist");
  strcpy(help[22].key,"HOME");
  strcpy (help[22].desc, "Jump to currently playing song");
  strcpy(help[23].key,"Q");
  strcpy (help[23].desc, "Quit");
  strcpy(help[24].key,"H");
  strcpy (help[24].desc, "Show this help screen");
  strcpy(help[25].key,"D");
  strcpy (help[25].desc, "Delete selected song");
  strcpy(help[26].key,"[");
  strcpy (help[26].desc, "Move selected song up");
  strcpy(help[27].key,"]");
  strcpy (help[27].desc, "Move selected song down");
  strcpy(help[28].key, "T");
  strcpy (help[28].desc, "Goto top of playlist");
  strcpy(help[29].key, "C");
  strcpy (help[29].desc, "Edit Configuration");
  halfdelay (255);
  getmaxyx(stdscr, row, col);
  help_win = newwin (row - 3, col - 10, 2, 5);
  werase (help_win);
  wbkgd (help_win, COLOR_PAIR (2) | A_BOLD);
  box (help_win, 0, 0);
  rows = row - 8;
  while (1)
    {
      for (i = 0; i < rows && i < help_rows; i++)
	{
	  mvwprintw (help_win, i + 2, 5, "%s", help[i + current].key);
	  mvwprintw (help_win, i + 2, 13, "%s", help[i + current].desc);
	}
      mvwprintw (help_win, rows + 3, 5,
		 "Use arrows to move up/down and Q to close help screen");
      wrefresh (help_win);
      c = toupper(getch()); //will work when capslock is on =)
      if (c == 'Q')
	break;
      else if (c == KEY_DOWN && help_rows - current > rows)
	current++;
      else if (c == KEY_UP && current)
	current--;
      werase (help_win);
      box (help_win, 0, 0);
    }
  halfdelay (1);
  delwin (help_win);
  doupdate();
}

void show_filebrowser ()
{
  int file_select ();
  int sort_dirs_first ();
  int file_top = 0, num_added_files = 0;
  char added_files[1000][256], tmp[256];
  WINDOW *filebrowse_win, *path_win;
  char pathname[MAXPATHLEN], olddir[MAXPATHLEN];
  int c, old;
  extern int alphasort ();
  int num_files, i, q;
  struct dirent **files;
  int file_selected = 0, found;
  char *tmpdir;
  char newpathname[27];
  char tmpfilename[col - 15];
  
  getmaxyx(stdscr, row, col);
  getcwd (pathname, MAXPATHLEN);
  filebrowse_win = newwin (row - 5, col - 10, 2, 5);
  werase (filebrowse_win);
  wbkgd (filebrowse_win, COLOR_PAIR (2));
  wattr_on (filebrowse_win, A_BOLD, NULL);
  box (filebrowse_win, 0, 0);
  num_files = scandir (pathname, &files, file_select, alphasort);
  while (1)
    {
      getmaxyx(stdscr, row, col);
      for (i = 0; i < row - 5; i++)
	{
	  wattr_on (filebrowse_win, A_BOLD, NULL);
	  box (filebrowse_win, 0, 0);
	  mvwprintw (filebrowse_win, 0, 3, " %s ", pathname);
          //below added so user knows how to add and quit file browser
          mvwprintw (filebrowse_win, 0, 3, "%s Spacebar=Add, /=Path Q=Quit", pathname);
	  if (i + file_top == file_selected)
	    wattr_on (filebrowse_win, COLOR_PAIR (4), NULL);
	  else
	    wattr_on (filebrowse_win, COLOR_PAIR (3), NULL);
          mvwhline (filebrowse_win, i + 1, 1, 32, col - 12);
	  if (i + file_top < num_files)
	    {
	      found = 0;
	      for (q = 0; q < num_added_files; q++)
		{
		  sprintf (tmp, "%s/%s", pathname,
			   files[i + file_top]->d_name);
		  if (strcmp ((char *) &added_files[q], (char *) &tmp) == 0)
		    {
		      found = 1;
		      break;
		    }
		}
	      if (found)
		wattr_on (filebrowse_win, A_BOLD, NULL);
	      else
		wattr_off (filebrowse_win, A_BOLD, NULL);
	      strcpy (tmpfilename, files[i + file_top]->d_name);
              tmpfilename[col - 16] = '\0';
	      if (files[i + file_top]->d_type == 4)
		{
		  mvwprintw (filebrowse_win, i + 1, 1, "-> %s", tmpfilename);
		}
	      else
		{
		  mvwprintw (filebrowse_win, i + 1, 1, "   %s", tmpfilename);
		}
	    }
	  wattr_on (filebrowse_win, COLOR_PAIR (3), NULL);
          mvwvline (filebrowse_win, 1, row - 12, 32, col - 5);
	  wattr_on (filebrowse_win, COLOR_PAIR (3), NULL);
	  mvwvline (filebrowse_win,
                  (int) ((float) file_selected / num_files * (row - 5)) +
                                 1, col - 12, 32, 1);
	  wattr_on (filebrowse_win, COLOR_PAIR (3), NULL);
	}
      wrefresh (filebrowse_win);
      c = toupper(getch ());  //whether uppercase or lowercase, value is the same
      if (c == KEY_DOWN)
	{
	  if (file_selected < num_files - 1)
	    file_selected++;
	}
      else if (c == KEY_UP)
	{
	  if (file_selected)
	    file_selected--;
	}
      else if (c == KEY_NPAGE)
	{
          if (file_selected + row - 5 < num_files)
	    {
	      file_selected += row - 5;
	      file_top += row - 5;
	    }
	  else
	    {
	      file_selected = num_files - 1;
	    }
	}
      else if (c == KEY_PPAGE)
	{
           if (file_top - (row - 5) >= 0)
	    {
	      file_selected -= (row - 5);
	      file_top -= (row - 5);
	    }
	  else
	    {
	      file_selected = file_top = 0;
	    }
	}
      else if (c == 10 || c == KEY_LEFT || c == KEY_RIGHT || c == '/')
	{
           if (c == '/')
            {
             getmaxyx(filebrowse_win,row,col);
             path_win = subwin (filebrowse_win, 6, 43, 8 + ((row - 17) / 2), (col - 43) / 2);
             curs_set (1);
             echo ();
             werase (path_win);
             wbkgd (path_win, COLOR_PAIR (3) | A_BOLD);
             box (path_win, 0, 0);
             mvwprintw (path_win, 2, 2, "Enter path:");
             if (strlen (searchfor))
                mvwprintw (path_win, 3, 2, "[ %s ]", searchfor);
             wattr_on (path_win, COLOR_PAIR (3), NULL);
             mvwhline (path_win, 2, 14, 32, 27);
             wrefresh (path_win);
             mvwgetnstr (path_win, 2, 14, newpathname, 26);
             chdir (newpathname);
            }
	  if (c == KEY_LEFT)
	    {
	      files[file_selected]->d_type = 4;
	      strcpy (files[file_selected]->d_name, "..");
              chdir (files[file_selected]->d_name);
	    }
          if (c == 10 || c == KEY_RIGHT)
             chdir (files[file_selected]->d_name);
        
          if (files[file_selected]->d_type == 4)
	    {
	      //chdir was here
	      werase (filebrowse_win);
	      wattr_on (filebrowse_win, A_BOLD, NULL);
	      box (filebrowse_win, 0, 0);
	      if (strncmp ((char *) &files[file_selected]->d_name, "..", 2) ==
		  0)
		{
		  tmpdir = rindex (pathname, '/');
		  strcpy ((char *) olddir, (char *) ++tmpdir);
		  old = 1;
		}
	      else
		{
		  old = 0;
		}
	      getcwd (pathname, MAXPATHLEN);
	      num_files = scandir (pathname, &files, file_select, alphasort);
	      if (old)
		{
		  i = 0;
		  while (i < num_files)
		    {
		      if (strcmp ((char *) olddir, (char *) files[i]->d_name)
			  == 0)
			break;
		      i++;
		    }
		  if (i == num_files)
		    file_selected = file_top = 0;
		  else
		    {
		      file_selected = i;
		      file_top = i - 5;
		    }
		}
	      else
		{
		  file_selected = file_top = 0;
		}
	    }

	}
      else if (c == 32)
	{
	  if (files[file_selected]->d_type != 4)
	    {
	      found = 0;
	      for (q = 0; q < num_added_files; q++)
		{
		  sprintf (tmp, "%s/%s", pathname,
			   files[file_selected]->d_name);
		  if (strcmp ((char *) &added_files[q], (char *) &tmp) == 0)
		    {
		      found = q + 1;
		      break;
		    }
		}

	      if (found)
		{
		  added_files[found - 1][0] = '\0';
		  if (file_selected < num_files - 1)
		    file_selected++;
		}
	      else
		{
		  sprintf (added_files[num_added_files], "%s/%s", pathname,
			   files[file_selected]->d_name);
		  num_added_files++;
		  if (file_selected < num_files - 1)
		    file_selected++;
		}
	    }
	  else
	    {
	      if (file_selected < num_files - 1)
		file_selected++;
	    }
	}

      else if (c == 'Q')
	{
	  for (q = 0; q < num_added_files; q++)
	    if (added_files[q][0] != '\0')
	      {
                load_playlist (added_files[q]);
		playlist_changes = 1;
	      }
	  if (random_play)
	    generate_random_playlist ();
	  break;
	}

      if (file_selected >= file_top + row - 5)
	file_top += row - 5;
      else if (file_selected < file_top)
          file_top -= row -5;
      if (file_top < 0)
	file_top = 0;
    }
  chdir (startdir);
  //above to ensure we are in the correct directory for saving playlist.
  //wclear (stdscr);
  delwin (filebrowse_win);
  doupdate();
}

int file_select (struct dirent *entry)
{
  char *ptr;
  struct stat b;

  if (strcmp (entry->d_name, ".") == 0)
    return (0);

  ptr = rindex (entry->d_name, '.');
  if ((ptr != NULL && strcasecmp (ptr, ".mp3") == 0)
      || (ptr != NULL && strcasecmp (ptr, ".m3u") == 0)
      || (ptr != NULL && strcasecmp (ptr, ".mkq") == 0) 
      || (ptr != NULL && strcasecmp (ptr, ".ogg") == 0))
    return (1);

  stat ((char *) entry->d_name, (struct stat *) &b);
  if (S_ISDIR (b.st_mode))
    {
      entry->d_type = 4;
      return (1);
    }
  return (0);

}

int is_a_number(char *str)
{
     int i = 0, val_is_num = 0;
     while (isalnum(str[i]) != 0 && i < strlen(str)) 
        {    
            if (isdigit(str[i]) != 0) 
                { 
                    val_is_num = 1;
                } 
            else
                {
                    val_is_num = 0;
                    break;
                }
            i++; 
        }
     return(val_is_num);
}

void show_config_win(int use_cdrom, int buffers, int channels, int vol)
{
    char *opt_value[8], *home, conffile[256], *line, *tmpstr, *tmpstr2;
    int ch, x, opt_num, options[8], doexit;
    int crow, ccol, i, val_is_num;
    FILE * infile;
    WINDOW *config_win;
    
    //0 random
    //1 loop
    //2 cdrom
    //3 default vol
    //4 xterm title
    //5 delay between songs
    //6 random method
    //7 buffers
    //8 channel
    
    doexit = 0;
    opt_num = 0;
    getmaxyx(stdscr, row, col);
    config_win = newwin (25, col / 3, 5, col - (col / 2) - 10);
    getmaxyx(config_win, crow, ccol);
    wbkgd(config_win, COLOR_PAIR (2));
    werase (config_win);
    if (random_play)
        opt_value[0] = "Y";
    else opt_value[0] = "N";
    if (repeat_play)
        opt_value[1] = "Y";
    else opt_value[1] = "N";
    if (use_cdrom)
        opt_value[2] = "Y";
    else opt_value[2] = "N";
    opt_value[3] = itoa(vol);
    if (xterm_title)
        opt_value[4] = "Y";
    else opt_value[4] = "N";
    if (delay_between_songs)
        opt_value[5] = itoa(delay_between_songs);
    else {
            opt_value[5][0] = '0';
            opt_value[5][1] = '\0';
         }
    if (random_method)
        opt_value[6] = itoa(random_method);
    else opt_value[6] = "0";
    opt_value[7] = itoa(buffers);
    opt_value[8] = itoa(channels);
    
    while(!doexit)
    {
        wattr_on (config_win, COLOR_PAIR (3) | A_BOLD, NULL);
        mvwprintw(config_win, 1, (ccol / 2) - 11, "Config Menu");
        for (x = 0; x < 10; x++)
        {
            switch(x)
              {
                case 0:
                    if (opt_num == 0)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 3, 2, "Random? [%s]", 
                                    opt_value[x]);
                            wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                    else mvwprintw (config_win, 3, 2, "Random? [%s]", 
                                    opt_value[x]);
                    break;
                case 1:
                    if (opt_num == 1)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 5, 2, "Loop? [%s]", 
                                    opt_value[x]);
                            wattr_on (config_win, COLOR_PAIR (3), NULL);
                        } 
                    else mvwprintw (config_win, 5, 2, "Loop? [%s]", 
                                    opt_value[x]);
                    break;
                case 2:
                    if (opt_num == 2)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 7, 2, "Use CDROM? [%s]", 
                                    opt_value[x]);
                            wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                    else mvwprintw (config_win, 7, 2, "Use CDROM? [%s]", 
                                    opt_value[x]);
                    break;
                case 3:
                      if (opt_num == 3)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 9, 2, "Default Volume [ %s ]", 
                                    opt_value[x]);
                            wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                      else mvwprintw (config_win, 9, 2, "Default Volume [ %s ]", 
                                    opt_value[x]);
                    break;
                case 4:
                    if (opt_num == 4)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 11, 2, "Set XTerm Title? [%s]", 
                                    opt_value[x]);
                              wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                      else mvwprintw (config_win, 11, 2, "Set XTerm Title? [%s]", 
                                    opt_value[x]);
                    break;
                case 5:
                     if (opt_num == 5)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 13, 2, "Delay Between Songs [ %s ]", 
                                    opt_value[x]);
                              wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                     else  mvwprintw (config_win, 13, 2, "Delay Between Songs [ %s ]", 
                                    opt_value[x]);
                    break;
                case 6:
                    if (opt_num == 6)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 15, 2, "Random Method [%s]", 
                                    opt_value[x]);
                              wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                    else mvwprintw (config_win, 15, 2, "Random Method [%s]", 
                                    opt_value[x]);
                    break;
                case 7:
                     if (opt_num == 7)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 17, 2, "Audio Buffers [ %s ]", 
                                    opt_value[x]);
                              wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                     else mvwprintw (config_win, 17, 2, "Audio Buffers [ %s ]", 
                                    opt_value[x]);
                    break;
                case 8:
                    if (opt_num == 8)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 19, 2, "Audio Channels [%s]", 
                                    opt_value[x]);
                              wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                    else mvwprintw (config_win, 19, 2, "Audio Channels [%s]", 
                            opt_value[x]);
                    break;
                case 9:
                     if (opt_num == 9)
                        {
                            wattr_on (config_win, COLOR_PAIR (1) | A_BOLD, NULL);
                            mvwprintw (config_win, 21, 2, "Done");
                            wattr_on (config_win, COLOR_PAIR (3), NULL);
                        }
                     else mvwprintw (config_win, 21, 2, "Done");
                    break;
            }
        }
        wattr_on (config_win, COLOR_PAIR (2), NULL);
        box(config_win, 0, 0);
        wrefresh(config_win);
        wrefresh(main_win);
        doupdate();
        ch = toupper(getch());
        doupdate();
        switch(ch)
            {
                case KEY_DOWN:
                    opt_num++;
                    if (opt_num > 9)
                        opt_num = 0;
                    break;
                case KEY_UP:
                    opt_num--;
                    if (opt_num < 0)
                        opt_num = 9;
                    break;
                case 10:
                    switch(opt_num)
                        {
                            case 0:
                                if (random_play)
                                    {
                                        random_play = 0;
                                        opt_value[0] = "N";
                                    }
                                else 
                                    { 
                                        random_play =1;
                                        opt_value[0] = "Y";
                                    }
                                break;
                            case 1:
                                if (repeat_play)
                                    {
                                        repeat_play = 0;
                                        opt_value[1] = "N";
                                    }
                                else 
                                    { 
                                        repeat_play =1;
                                        opt_value[1] = "Y";
                                    }
                                break;
                            case 2:
                                if (use_cdrom)
                                    {
                                        use_cdrom = 0;
                                        opt_value[2] = "N";
                                    }
                                else 
                                    { 
                                        use_cdrom =1;
                                        opt_value[2] = "Y";
                                    }
                                break;
                            case 3:
                                wattr_on (config_win, COLOR_PAIR (3), NULL);
                                mvwhline (config_win, 9, 19, 32, 4);
                                wrefresh (config_win);
                                echo();
                                mvwgetnstr (config_win, 9, 19, opt_value[3], 3);
                                if (strlen(opt_value[3]) == 0)
                                    opt_value[3] = itoa(vol);
                                else 
                                    { 
                                    val_is_num = is_a_number(opt_value[3]);
                                    if (atoi(opt_value[3]) >= 0 && 
                                            atoi(opt_value[3]) < 100 && 
                                            val_is_num)
                                        vol = atoi(opt_value[3]);
                                    else opt_value[3] = itoa(vol);
                                    }
                                noecho();
                                break;
                            case 4:
                                if (xterm_title)
                                    {
                                        xterm_title = 0;
                                        opt_value[4] = "N";
                                    }
                                else 
                                    { 
                                        xterm_title =1;
                                        opt_value[4] = "Y";
                                    }
                                break;
                            case 5:
                                wattr_on (config_win, COLOR_PAIR (3), NULL);
                                mvwhline (config_win, 13, 23, 32, 3);
                                wrefresh (config_win);
                                echo();
                                mvwgetnstr (config_win, 13, 23, opt_value[5], 2);
                                if (strlen(opt_value[5]) == 0)
                                    opt_value[5] = itoa(delay_between_songs);
                                else 
                                    { 
                                    val_is_num = is_a_number(opt_value[5]);
                                    if (atoi(opt_value[5]) >= 0 && 
                                            atoi(opt_value[5]) < 99 && 
                                            val_is_num)
                                        delay_between_songs = atoi(opt_value[5]);
                                    else opt_value[5] = itoa(delay_between_songs);
                                    }
                                noecho();
                                break;
                            case 6:
                                wattr_on (config_win, COLOR_PAIR (3), NULL);
                                mvwhline (config_win, 15, 17, 32, 2);
                                wrefresh (config_win);
                                echo();
                                mvwgetnstr (config_win, 15, 17, opt_value[6], 1);
                                if (strlen(opt_value[6]) == 0)
                                    opt_value[6] = itoa(random_method);
                                else 
                                    {
                                        if (atoi(opt_value[6]) > 0 && 
                                                atoi(opt_value[6]) < 4)
                                            random_method = atoi(opt_value[6]);
                                        else opt_value[6] = itoa(random_method);
                                    }
                                noecho();
                                break;
                            case 7:
                                wattr_on (config_win, COLOR_PAIR (3), NULL);
                                mvwhline (config_win, 17, 17, 32, 5);
                                wrefresh (config_win);
                                echo();
                                mvwgetnstr (config_win, 17, 17, opt_value[7], 4);
                                if (strlen(opt_value[6]) == 0)
                                    opt_value[7] = itoa(buffers);
                                else 
                                    { 
                                    val_is_num = is_a_number(opt_value[7]);
                                    if (atoi(opt_value[7]) >= 0 && 
                                            atoi(opt_value[7]) < 4097 && 
                                            val_is_num)
                                        buffers = atoi(opt_value[7]);
                                    else opt_value[7] = itoa(buffers);
                                    }
                                noecho();
                                break;
                            case 8:
                                 wattr_on (config_win, COLOR_PAIR (3), NULL);
                                mvwhline (config_win, 19, 18, 32, 2);
                                wrefresh (config_win);
                                echo();
                                mvwgetnstr (config_win, 19, 18, opt_value[8], 1);
                                if (strlen(opt_value[8]) == 0)
                                    opt_value[7] = itoa(channels);
                                else 
                                    {   
                                    if (atoi(opt_value[8]) > 0 && 
                                            atoi(opt_value[8]) < 9)
                                        channels = atoi(opt_value[8]);
                                    else opt_value[8] = itoa(channels);
                                    }
                                noecho();
                                break;
                            case 9:
                                doexit = 1;
                                break;
                        }
                    break;
                case 'Q':
                    doexit = 1;
                    break;
        }
    werase (config_win);
    doupdate();
    }
  home = getenv ("HOME");
  sprintf ((char *) &conffile, "%s/.muzikQrc", home);
  infile = fopen (conffile, "w");
  if (infile)
    {
      if (use_cdrom)
        {
            line = "cdrom = 1";  
            fprintf(infile, "%s\n", line);
        }
      if (delay_between_songs)
        {
          tmpstr = "delay_between_songs = ";
          fputs(tmpstr, infile);
          line = itoa(delay_between_songs);
          fprintf(infile, "%s\n", line);
        }
      if (random_method)
        {
          tmpstr = "random_method = ";
          fputs(tmpstr, infile);
          line = itoa(random_method);
          fprintf(infile, "%s\n", line);
        }
      if (random_play)
        {
          line = "random = 1";
          fprintf(infile, "%s\n", line);
        }
      if (xterm_title)
        {
          line = "set_xterm_title = 1";
          fprintf(infile, "%s\n", line);
        }
      if (repeat_play)
        {
          line = "loop = 1";
          fprintf(infile, "%s\n", line);
        }
      if (vol)
         {
          tmpstr = "volume = \0";
          fputs(tmpstr, infile);
          line = itoa(vol);
          fprintf(infile, "%s\n", line);
        } 
      if (buffers >= 0)
        {
          tmpstr = "buffers = \0";
          fputs(tmpstr, infile);
          line = itoa(buffers);
          fprintf(infile, "%s\n", line);
        }
      if (channels > 0)
        {
          tmpstr = "channels = \0";
          fputs(tmpstr, infile);
          line = itoa(channels);
          fprintf(infile, "%s\n", line);
        }
      fclose(infile);
    }
  halfdelay (1);
  wclear (stdscr);
  delwin (config_win);
  doupdate();
}

void repaint ()
{
  endwin ();
  refresh();
  getmaxyx(stdscr, row, col);
  resizeterm(row, col);
  init_screen ();
  doupdate();
}